@@ -10,6 +10,8 @@ urlpatterns = [
     path('select-company', auth_views.SelectCompanyView.as_view(), name='select_company'),
     path('set-company/<uuid:company_id>', auth_views.SetCompanyView.as_view(), name='set_company'),
     path('webhook', auth_views.WebhookView.as_view(), name='webhook'),
+    path('impersonation', auth_views.ImpersonationCheckView.as_view(), name='impersonation-check'),
+    path('impersonation/<uuid:token>', auth_views.ImpersonationView.as_view(), name='impersonation-setup'),
 ]
 
 # Serving media files in development
