@@ -165,6 +165,55 @@ class SetCompanyView(LoginRequiredMixin, View):
         if not self.company: return HttpResponseBadRequest(_('The selected company does not exist'))
         return super().dispatch(request, *args, **kwargs)
 
+class ImpersonationCheckView(View):
+    
+    def get(self, request, *args, **kwargs):
+        return JsonResponse({ 'status' : 'operational' })
+
+class ImpersonationView(View):
+    """ Handles the verification and authentication for an impersonation request """
+    
+    def get(self, request, *args, **kwargs):
+        redirect_url = settings.LOGIN_REDIRECT_URL if hasattr(settings, 'LOGIN_REDIRECT_URL') else '/'
+        
+        impersonation_token = kwargs.get('token', None)
+        api = CloudspotLicense_API(settings.CLOUDSPOT_LICENSE_APP)
+        
+        validate = api.auth.validate_impersonation(impersonation_token)
+        if validate.has_error:
+            return HttpResponseBadRequest('Token could not be validated. Error: {0}'.format(validate.error.message))
+
+        # user and company checking
+        user_id = validate.user
+        company_id = validate.company
+        auth_token = validate.auth_token
+        
+        user = get_user_model().objects.filter(pk=user_id).first()
+        if not user: return HttpResponseBadRequest('User with ID {0} could not be found.'.format(user_id))
+        company = CloudspotCompany.objects.filter(pk=company_id).first()
+        if not company: return HttpResponseBadRequest('Company with ID {0} could not be found.'.format(company_id))
+        if company not in user.available_companies.all(): return HttpResponseBadRequest('User is not in requested company.')
+        
+        # User is a valid user and has a valid company
+        # Use the returned auth token to get all the permissions for the specified company
+        api = CloudspotLicense_API(settings.CLOUDSPOT_LICENSE_APP, token=auth_token)
+        
+        user.company = company
+        user.save()
+        
+        try:
+            permissions = api.get_company_permissions(company_id)
+        except Exception as e:
+            return HttpResponseBadRequest('Could not retrieve permissions for user. Error: {0}'.format(e))
+        
+        # Assign all the returned permissions
+        grant_permissions(user, permissions)
+        
+        # Login
+        messages.success(request, f"You're now impersonating <b>{user.get_full_name()}.")
+        login(request, user)
+        return redirect(redirect_url)
+
 @method_decorator(csrf_exempt, name='dispatch')
 class WebhookView(View):
     """ Handles all the webhook events that the Cloudspot License Server server sends """
@@ -186,19 +235,31 @@ class WebhookView(View):
             
             permissions = data['permissions']
             company_id = data['company_id']
+            available_companies = data['available_companies']
+            
+            # update the companies that the user is in
+            user.available_companies.clear()
+            for company_perm in available_companies:
+                company = CloudspotCompany.objects.filter(id=company_perm['company_id']).first()
+                if not company: company = CloudspotCompany.objects.create(id=company_perm['company_id'], name=company_perm['company_name'])
+                elif company.name != company_perm['company_name']:
+                    company.name = company_perm['company_name']
+                    company.save()
+                
+                user.available_companies.add(company.id)
             
             try:
                 company = CloudspotCompany.objects.get(pk=company_id)
             except CloudspotCompany.DoesNotExist:
                 return JsonResponse({ 'status' : ResponseStatus.NOT_FOUND, 'error' : { 'message' : 'No company matched the id.' }}, status=HTTPStatus.NOT_FOUND)
-            
+
             # Only update the user permissions if the current company is the company in the request
             # Otherwise, these permissions do not apply to this user at this moment
             # When the user switches companies, the new permissions get retrieved from the License Server anyway
             if user.company == company:
                 if 'use_app' not in permissions:
                     # if the 'use_app' permission is revoked, the user is not allowed to use the app for this company anymore.
-                    # we set the company to none. This will force the user to choose a new company on next page load.
+                    # we set the company to none. This will force the user to choose a new company on next page load or get logged out.
                     user.company = None
                     user.save()
                 else:
